/*****************************************************
 * tnhScript.cc was getting crowded
 * I'll put the hack/patch scripts here.
 */

#include <cstdlib>
#include <cstring>
#include <cctype>

#include "lg/interface.h"
#include "lg/scrservices.h"
#include "lg/scrmanagers.h"
#include "lg/scrmsgs.h"
#include "lg/properties.h"
#include "lg/propdefs.h"
#include "lg/gen.h"
#include "ScriptLib.h"
#include "tnhScript.h"


/******************
 * NewKey
 */
cScr_NewKey::~cScr_NewKey()
{
	if (m_pStdKey)
		m_pStdKey->Release();
}

void cScr_NewKey::InitScriptData()
{
	const sScrClassDesc* scr = g_pScriptManager->GetClass("StdKey");
	if (scr)
		m_pStdKey = (*scr->pfnFactory)(scr->pszClass, m_iObjId);

	cScript::InitScriptData();
	RegisterMessageHandler("FrobToolEnd", static_cast<MessageHandlerFunc>(&cScr_NewKey::OnFrobToolEnd));
}

long __stdcall cScr_NewKey::ReceiveMessage(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	long iRet = cScript::ReceiveMessage(pMsg, pReply, eTrace);
	if (m_pStdKey && iRet) {
		iRet = m_pStdKey->ReceiveMessage(pMsg, pReply, eTrace);
	}
	return iRet;
}

long __stdcall cScr_NewKey::OnFrobToolEnd(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	sFrobMsg* pFrobMsg = static_cast<sFrobMsg*>(pMsg);
	SService<IKeySrv> pKeySrv(g_pScriptManager);
	int iSuccess = pKeySrv->TryToUseKey(m_iObjId, pFrobMsg->DstObjId, kKeyUseCheck);
	SService<IObjectSrv> pObjSrv(g_pScriptManager);
	object oPlayer;
	pObjSrv->Named(oPlayer, "Player");
	if (iSuccess)
	{
		if (pFrobMsg->Frobber == oPlayer)
		{
			SimpleSend(m_iObjId, pFrobMsg->DstObjId, "PlayerToolFrob");
			SService<ILockSrv> pLockSrv(g_pScriptManager);
			SService<IQuestSrv> pQSrv(g_pScriptManager);
			if (pLockSrv->IsLocked(pFrobMsg->DstObjId))
				pQSrv->Set("tr_UseKeyLock", 1, 0);
			else
				pQSrv->Set("tr_UseKeyUnlock", 1, 0);
		}
		// This is the bug. GEN.OSM calls TryToUseKey before sending PlayerToolFrob.
		pKeySrv->TryToUseKey(m_iObjId, pFrobMsg->DstObjId, kKeyUseDefault);
		SService<IDamageSrv> pDSrv(g_pScriptManager);
		pDSrv->Slay(m_iObjId, pFrobMsg->Frobber);
	}
	else // !iSuccess
	{
		SService<ILockSrv> pLockSrv(g_pScriptManager);
		SService<IPropertySrv> pPropSrv(g_pScriptManager);
		if (( pPropSrv->Possessed(pFrobMsg->DstObjId, "RotDoor")
		   || pPropSrv->Possessed(pFrobMsg->DstObjId, "TransDoor"))
		  && !pPropSrv->Possessed(pFrobMsg->DstObjId, "Locked")
		  && !pLockSrv->IsLocked(pFrobMsg->DstObjId))
		{
			if (pFrobMsg->Frobber == oPlayer)
			{
				SimpleSend(m_iObjId, pFrobMsg->DstObjId, "PlayerToolFrob");
			}
			SService<IDoorSrv> pDoorSrv(g_pScriptManager);
			pDoorSrv->ToggleDoor(pFrobMsg->DstObjId);
			iSuccess = 1;
		}
		else
		{
			SimpleSend(m_iObjId, pFrobMsg->DstObjId, "Wrongkey", m_iObjId);
		}
	}
	if (pReply)
		MultiParmAssignInt(pReply, iSuccess);
	return 0;
}


/*******************
 * NewGasLight
 */

cScr_NewGasLight::~cScr_NewGasLight()
{
	if (m_pGasLight)
		m_pGasLight->Release();
}

void cScr_NewGasLight::InitScriptData()
{
	const sScrClassDesc* scr = g_pScriptManager->GetClass("GasLight");
	if (scr)
		m_pGasLight = (*scr->pfnFactory)(scr->pszClass, m_iObjId);

	cScript::InitScriptData();
	RegisterMessageHandler("FrobWorldEnd", static_cast<MessageHandlerFunc>(&cScr_NewGasLight::OnFrobWorldEnd));
	RegisterMessageHandler("TurnOn", static_cast<MessageHandlerFunc>(&cScr_NewGasLight::OnTurnOff));
	RegisterMessageHandler("TurnOff", static_cast<MessageHandlerFunc>(&cScr_NewGasLight::OnTurnOff));
	RegisterMessageHandler("FireStimStimulus", static_cast<MessageHandlerFunc>(&cScr_NewGasLight::OnTurnOff));
	RegisterMessageHandler("WaterStimStimulus", static_cast<MessageHandlerFunc>(&cScr_NewGasLight::OnExtinguish));
	RegisterMessageHandler("KOGasStimulus", static_cast<MessageHandlerFunc>(&cScr_NewGasLight::OnExtinguish));
}

long __stdcall cScr_NewGasLight::ReceiveMessage(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	long iRet = cScript::ReceiveMessage(pMsg, pReply, eTrace);
	if (m_pGasLight && iRet) {
		iRet = m_pGasLight->ReceiveMessage(pMsg, pReply, eTrace);
	}
	RemoveSingleMetaProperty("FrobInert", m_iObjId);
	return iRet;
}

long __stdcall cScr_NewGasLight::OnFrobWorldEnd(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	if (ToggleEmitter(kTweqDoHalt))
		return 0;
	true_bool bOn;
	if (*_DoScriptCall1a(IScr_AnimLight::IsLightOn,static_cast<IScr_AnimLight*>(m_pGasLight),&bOn))
	{
		sScrMsg offMsg;
		offMsg.message = "TurnOff";
		offMsg.from = pMsg->from;
		offMsg.to = pMsg->to;
		offMsg.time = pMsg->time;
		m_pGasLight->ReceiveMessage(&offMsg, pReply, eTrace);
		return 0;
	}
	return 1;
}

long __stdcall cScr_NewGasLight::OnTurnOff(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	ToggleEmitter(kTweqDoHalt);
	return 1;
}

long __stdcall cScr_NewGasLight::OnExtinguish(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	true_bool bOn;
	if (*_DoScriptCall1a(IScr_AnimLight::IsLightOn,static_cast<IScr_AnimLight*>(m_pGasLight),&bOn))
	{
		ToggleEmitter(kTweqDoActivate);
	}
	return 1;
}

int cScr_NewGasLight::ToggleEmitter(int iAction)
{
	SService<IPropertySrv> pPropSrv(g_pScriptManager);
	if (!pPropSrv->Possessed(m_iObjId, "StTweqEmit"))
		return 0;
	int iRet = 0;
	cMultiParm mpState;
	pPropSrv->Get(mpState, m_iObjId, "StTweqEmit", "AnimS");
	iRet = static_cast<int>(mpState) & kTweqFlagOn;
	SService<IActReactSrv> pARSrv(g_pScriptManager);
	cMultiParm& mpNull = const_cast<cMultiParm&>(g_mpUndef);
	cMultiParm mpType = kTweqTypeEmitter;
	cMultiParm mpAction = iAction;
	pARSrv->React(pARSrv->GetReactionNamed("tweq_control"), 1.0, m_iObjId, 0,
			  mpType, mpAction, mpNull, mpNull, mpNull, mpNull, mpNull, mpNull);
	return iRet;
}

