//#include <fstream>
#include <cstring>
#include <windows.h>
#include <winver.h>

#include "lg/objstd.h"
#include "lg/interface.h"

#include "lg/types.h"
#include "lg/defs.h"
#include "lg/scrmsgs.h"
#include "lg/scrservices.h"
#include "lg/objects.h"
#include "lg/input.h"

#include "ScriptLib.h"
#include "ScriptModule.h"
#include "scrversion.h"

//static bool UseDarkInst(void);
static char* GetCfgFromFile(const char* pszName, const char* pszFile);
static char* GetScriptPaths(void);
static char* FindScriptModule(const char* pszName, const char* pszPaths);
static void DoSuccess(int iObjId);
static void DoFailure(int iObjId);

using namespace std;


const sScrClassDesc cScriptModule::sm_ScriptsArray[] = {
	{ "version", "VersionCheck", "CustomScript", cScr_VersionCheck::MakeVersionCheck },
};
const unsigned int cScriptModule::sm_ScriptsArraySize = 1;


IScript* cScr_VersionCheck::MakeVersionCheck(const char* pszName, int iHostObjId)
{
	cScr_VersionCheck* pscrRet = new(nothrow) cScr_VersionCheck(pszName, iHostObjId);
	if (pscrRet)
	{
		pscrRet->AddRef();
		pscrRet->InitScriptData();
	}
	return static_cast<IScript*>(pscrRet);
}

long __stdcall cScr_VersionCheck::OnTimer(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	if (!stricmp(static_cast<sScrTimerMsg*>(pMsg)->name, "ErrorText"))
	{
		ShowBook(m_iObjId, true, 0x7FFFFFFFUL);
		SInterface<IObjectSystem> pOS(g_pScriptManager);
		pOS->Destroy(m_iObjId);
		return 0;
	}
	return 1;
}

long __stdcall cScr_VersionCheck::OnSim(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	if (!static_cast<sSimMsg*>(pMsg)->fStarting)
		return 1;

	char* pszScriptPaths = GetScriptPaths();
	char* pszParams = GetObjectParams(m_iObjId);
	char* pszScript;
	char* pszToken = pszParams;
	for (pszScript = strsep(&pszToken, ";"); pszScript; pszScript = strsep(&pszToken, ";"))
	{
		if (!*pszScript)
			continue;
		DWORD scrVersHigh = 0, scrVersLow = 0;
		char* pszVers = strchr(pszScript, '=');
		if (pszVers)
		{
			char* pt = pszVers + 1;
			scrVersHigh = strtoul(pt, &pt, 10) << 16;
			if (pt && *pt == '.')
			{
				scrVersHigh |= strtoul(pt+1, &pt, 10);
				if (pt && *pt == '.')
				{
					scrVersLow = strtoul(pt+1, &pt, 10) << 16;
					if (pt && *pt == '.')
						scrVersLow |= strtoul(pt+1, &pt, 10);
				}
			}
			*pszVers = '\0';
		}
		char* pszScriptFile = FindScriptModule(pszScript, pszScriptPaths);
		if (!pszScriptFile)
		{
			DoFailure(m_iObjId);
			g_pMalloc->Free(pszParams);
			return 1;
		}
		if (scrVersHigh == 0 && scrVersLow == 0)
		{
			delete[] pszScriptFile;
			continue;
		}
		DWORD z;
		unsigned int len = ::GetFileVersionInfoSizeA(pszScriptFile, &z);
		if (len)
		{
			char* pBuffer;
			VS_FIXEDFILEINFO* pFileVers;
			pBuffer = new(nothrow) char[len];
			if (pBuffer)
			{
				::GetFileVersionInfoA(pszScriptFile, z, len, reinterpret_cast<void*>(pBuffer));
				len = 0;
				::VerQueryValueA(reinterpret_cast<void*>(pBuffer), "\\", reinterpret_cast<void**>(&pFileVers), &len);
				if (len > 0)
				{
					if ( (pFileVers->dwFileVersionMS < scrVersHigh)
					  || (pFileVers->dwFileVersionMS == scrVersHigh 
					   && pFileVers->dwFileVersionLS < scrVersLow)
					)
					{
						DoFailure(m_iObjId);
						delete[] pBuffer;
						delete[] pszScriptFile;
						g_pMalloc->Free(pszParams);
						if (pszScriptPaths)
							delete[] pszScriptPaths;
						return 1;
					}
				}
				delete[] pBuffer;
			}
		}
		else
		{
			DoFailure(m_iObjId);
			delete[] pszScriptFile;
			g_pMalloc->Free(pszParams);
			if (pszScriptPaths)
				delete[] pszScriptPaths;
			return 1;
		}
		delete[] pszScriptFile;
	}
	DoSuccess(m_iObjId);
	g_pMalloc->Free(pszParams);
	if (pszScriptPaths)
		delete[] pszScriptPaths;
	return 1;
}

/* Incompatibilities in the lg interfaces prevent us from 
 * building a universal OSM, so this function is moot.
bool UseDarkInst(void)
{
	bool p = false;
	char* gametype = GetCfgFromFile("game", "cam.cfg");
	if (gametype)
	{
		if (!stricmp(gametype, "dark"))
		{
			if (::GetFileAttributesA("darkinst.cfg") != 0xFFFFFFFFUL)
				p = true;
		}
		delete[] gametype;
	}
	return p;
}
*/

char* GetCfgFromFile(const char* pszName, const char* pszFile)
{
	char buffer[140];
	char* value = NULL;
	HANDLE hCfgFile = ::CreateFileA(pszFile, GENERIC_READ, FILE_SHARE_READ, NULL, 
					OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hCfgFile != INVALID_HANDLE_VALUE)
	{
		DWORD dwCfgLen = strlen(pszName);
		DWORD dwBytes = 0;
		DWORD dwAvail = sizeof(buffer);
		char* startp = buffer;
		while (::ReadFile(hCfgFile, startp, dwAvail-1, &dwBytes, NULL))
		{
			char* line = buffer;
			char* endp = startp + dwBytes;
			*endp = '\0';
			while (line < endp)
			{
				char* endl = strchr(line, '\n');
				if (!endl)
				{
					if (dwBytes != 0)
					{
						dwBytes = endp - line;
						memmove(buffer, line, dwBytes);
						startp = buffer + dwBytes;
						dwAvail = sizeof(buffer) - dwBytes;
						goto parseCfgContinue;
					}
				}
				else
					*endl++ = '\0';

				if (!strnicmp(line,pszName,dwCfgLen) && isspace(line[dwCfgLen]))
				{
					char* l = &line[dwCfgLen+1];
					while (isspace(*l)) ++l;
					char* k = l + strlen(l) - 1;
					while (isspace(*k)) --k;
					++k;
					value = new(nothrow) char[k - l + 1];
					if (value)
					{
						strncpy(value,l,k - l);
						value[k-l] = '\0';
					}
					goto parseCfgEnd;
				}

				if ((line = endl) == NULL)
					break;
			}

			if (dwBytes == 0)
				break;
			dwAvail = sizeof(buffer);
			startp = buffer;
	parseCfgContinue:
			continue;
		}
	parseCfgEnd:
		CloseHandle(hCfgFile);
	}
	return value;
}

// script_module_path
char* GetScriptPaths(void)
{
	char* paths = NULL;
	// It's never easy...
	//char filepath[140];
	//filepath[0] = '\0';
	//int cwdlen = ::GetCurrentDirectoryA(126, filepath);
	//if (cwdlen > 0 && cwdlen < 126)
	//	strcat(filepath, "\\");
	// Ignore the path if it's really long, for whatever bizarre reason
	/*
	if (UseDarkInst())
		strcat(filepath, "darkinst.cfg");
	else
		strcat(filepath, "install.cfg");
	*/
#if (_DARKGAME == 2)
	paths = GetCfgFromFile("script_module_path", "darkinst.cfg");
#else
	paths = GetCfgFromFile("script_module_path", "install.cfg");
#endif
	return paths;
}

char* FindScriptModule(const char* pszName, const char* pszPaths)
{
	char filepath[160];
	if (!pszPaths || !*pszPaths)
	{
		strcpy(filepath, pszName);
		if (!strchr(filepath, '.'))
		{
			strcat(filepath, ".osm");
		}
		if (::GetFileAttributesA(filepath) != 0xFFFFFFFFUL)
		{
			char* ret = new(nothrow) char[strlen(filepath)+1];
			if (ret)
				strcpy(ret,filepath);
			return ret;
		}
		return NULL;
	}
	char *scrpaths = new char[strlen(pszPaths)+1];
	strcpy(scrpaths,pszPaths);
	char* tok = scrpaths;
	for (char* path = strsep(&tok, "+"); path; path = strsep(&tok, "+"))
	{
		while (isspace(*path)) ++path;
		if (!*path)
			continue;
		strcpy(filepath, path);
		strcat(filepath, "\\");
		strcat(filepath, pszName);
		if (!strchr(pszName, '.'))
			strcat(filepath, ".osm");
		if (::GetFileAttributesA(filepath) != 0xFFFFFFFFUL)
		{
			delete[] scrpaths;
			char* ret = new(nothrow) char[strlen(filepath)+1];
			if (ret)
				strcpy(ret,filepath);
			return ret;
		}
	}
	delete[] scrpaths;
	return NULL;
}

void DoSuccess(int iObjId)
{
	CDSend("TurnOn", iObjId);
	SInterface<IObjectSystem> pOS(g_pScriptManager);
	pOS->Destroy(iObjId);
}

void DoFailure(int iObjId)
{
	CDSend("TurnOff", iObjId);
	g_pScriptManager->SetTimedMessage2(iObjId, "ErrorText", 288, kSTM_OneShot, 0);
	//ShowBook(iObjId, true, 0x7FFFFFFFUL);
	//SInterface<IObjectSystem> pOS(g_pScriptManager);
	//pOS->Destroy(iObjId);
}

